import {SimpleTemplateState} from "./states/SimpleTemplateState";
import {Gamesystem} from "./Gamesystem";
import {SimpleState} from "./states/SimpleState";
import {SimpleTemplateTransition} from "./transitions/SimpleTemplateTransition";
import {TemplateType} from "../TemplateType";

export class SimpleTemplateGamesystem<ReferenceType> extends Gamesystem<SimpleTemplateState<ReferenceType>, SimpleTemplateTransition<ReferenceType>> {
  createState(label: string, description: string): SimpleState | undefined {
    if(label == null) {
      return undefined;
    }

    if(description == null) {
      description = "";
    }

    const state = new SimpleTemplateState<ReferenceType>(label, description);
    if(this.states.find(s => s.stateLabel == label) == undefined) {
      this.states.push(state);
      return state;
    } else {
      return undefined
    }
  }

  createTransition(startingState: SimpleTemplateState<ReferenceType>, endingState: SimpleTemplateState<ReferenceType>): SimpleTemplateTransition<ReferenceType> | undefined {
    if((startingState == null || endingState == null) || startingState === endingState) {
      return undefined;
    }

    const transition = new SimpleTemplateTransition<ReferenceType>(startingState, endingState);
    if(this.transitions.find(t => t.startingState === startingState && t.endingState === endingState) == undefined) {
      this.transitions.push(transition)
      return transition;
    } else {
      startingState.removeOutgoingTransition(transition);
      endingState.removeIncomingTransition(transition);
      return undefined
    }
  }

  removeState(state: SimpleTemplateState<ReferenceType>): boolean {
    const updatedStates = this.states.filter(s => s !== state);
    const updated = updatedStates.length != this.states.length;
    this.states = updatedStates;

    this.transitions = this.transitions.filter(t => t.startingState !== state && t.endingState !== state);

    return updated;
  }

  addReferenceKey(reference: ReferenceType) {
    this.states.forEach(state => {
      if(!state.conditionMap.has(reference)) {
        state.conditionMap.set(reference, state.conditions.concat())
      }
    })

    this.transitions.forEach(transition => {
      if(!transition.conditions.has(reference)) {
        transition.conditions.set(reference, transition.scriptAccountConditions.concat())
      }

      if(!transition.actions.has(reference)) {
        transition.actions.set(reference, transition.scriptAccountActions.concat())
      }
    })
  }

}
